/*================================================================
==================================================================
=	Program obslugi wyswietlacza wektorowego ze zSTERrowanym RTC
=	wersja = 1.11
=	CPU = ATMega162 (8515)
=	Optymalizacja = s
==================================================================	
================================================================*/

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include "vector_disp.h"
#include "i2c.h"
#include "PCF8583.h"
#include "graphics.h"



//================================================================
//===== zmienne globalne =========================================
//================================================================
volatile unsigned char czas[7]; 				// g,g,m,m,s,"godziny" i "minuty" (liczba stopni dla zegara analogowego)

volatile unsigned char co_wyswietlic = 3;		// okresla aktualnie wyswietlana tresc 
//================================================================



//================================================================
//===== Opoznienia ===============================================
//================================================================
// opoznienie 3cykle*count + ustawienie timera
void delay(unsigned char __count)
{
	__asm__ volatile (
		"1: dec %0" "\n\t"
		"brne 1b"
		: "=r" (__count)
		: "0" (__count)
	);
}


void delay2(void)				// dokladnie 10 cykli
{
	NOP();
	NOP();
}
//================================================================



//================================================================
//===== Programowa obsluga klawiatury ============================
//================================================================
void klawiatura_init(void)
{
	DDRA &=~ 0x1F;								// PA0-PA5 = wejscie
}


void klawiatura(void)
{
	int i1;
	unsigned char i2=1;
	
	cli();										// wylacz przerwania
	
	if((PINA & 30) != 30)						// pierwszy przycisk nie brany pod uwage
	{
		switch(PINA & 31)						// odczyt stanu PA0 - PA4 
		{
			case 28:							// PA0 i PA1 = L
				i2 = 0;
				if(czas[0] == 1)				// zwieksz godziny10
					czas[0] = 0;
				else
					czas[0] = 1;
				break;
		
			case 26:							// PA0 i PA2 = L
				i2 = 0;
				czas[1]++;						// zwieksz godziny1
				if(czas[0] == 0)
				{
					if(czas[1] > 9)
						czas[1] = 0;
				}
				else
				{
					if(czas[1] > 2)
						czas[1] = 0;
				}
				break;
		
			case 22:							// PA0 i PA3 = L
				i2 = 0;
				czas[2]++;						// zwieksz minuty10
				if(czas[2] > 5)
					czas[2] = 0;
				break;
		
			case 14:							// PA0 i PA4 = L
				czas[3]++;						// zwieksz minuty1
				if(czas[3] > 9)
					czas[3] = 0;
				break;
		
			case 29:
				if((co_wyswietlic & 1) == 0)						
					co_wyswietlic |= 1;			// wlacz zegar analogowy
				else
					co_wyswietlic &=~ 1;		// wylacz zegar analogowy
				break;
				
			case 27:
				if((co_wyswietlic & 2) == 0)						
					co_wyswietlic |= 2;			// wlacz zegar cyfrowy maly
				else
					co_wyswietlic &=~ 2;		// wylacz zegar cyfrowy maly
				break;
				
			case 23:
				if((co_wyswietlic & 4) == 0)						
					co_wyswietlic |= 4;			// wlacz przewijany tekst
				else
					co_wyswietlic &=~ 4;		// wylacz przewijany tekst
				break;
				
			/*case 15:
				if((co_wyswietlic & 8) == 0)						
					co_wyswietlic |= 8;			// wlacz zegar cyfrowy duzy
				else
					co_wyswietlic &=~ 8;		// wylacz zegar cyfrowy duzy
				break;*/
		
			default: 
				break;
		}
		
		if(i2 == 0)
		{
			wyslij_godzine();
		
			czas[6] = (czas[2] * 10) + czas[3];	// minuty(10)x10 + minuty(1)
			czas[5] = (czas[1] + (czas[0] * 10));	// godziny (1 + 10)
			czas[5] = (czas[5] * 6) + czas[2];		// godziny * 6 + minuty(10)
		}
		
		for(i1=0;i1<30000;i1++)					// opoznienie niwelujace drgania stykow
			delay2();
	}

	sei();											// wlacz przerwania
}
//================================================================




int main (void)
{
	graph_init();								// inicjalizuj wyswietlanie grafiki

	klawiatura_init();							// inicjalizuj klawisze sterujace

	PCF8583_init();								// inicjalizuj RTC
	pobierz_godzine();							// pobierz godzine z RTC
	
	sei();										// globalne uruchomienie przerwan

	while(1)
	{
		klawiatura();							// sprawdz stan klawiatury
		
		if((co_wyswietlic & 1) == 1)			
			zegar_analogowy();
			
		if((co_wyswietlic & 2) == 2)	
			zegar_cyfrowy();
		
		if((co_wyswietlic & 4) == 4)
			tekst_rastrowy();
	}

return(0);
}



//================================================================
//===== "wygaszacz" ==============================================
//================================================================
/*void losuj_krzywe_beziera(void)
{
	signed char i1,i3;
	unsigned int i2;
	
	for(i1=0;i1<10;i1++)						// losowanie nowych wspolrzednych
	{
		i3 = 1;
		if(rand() > 32250)
			i3 = -1;
			
		i2 = bezier[i1][X] + (10*i3);		// dodanie od -4 do 4
		if(i2 > 255)
			i2 = 0;
		bezier[i1][X] = i2; 
		
		i3 = 1;
		if(rand() > 32250)
			i3 = -1;
		
		i2 = bezier[i1][Y] + (10*i3);		// dodanie od -4 do 4
		if(i2 > 255)
			i2 = 0;
		bezier[i1][Y] = i2; 
	}
	
	for(i1=0;i1<10;i1++)						// wyznaczenie przesuniec
	{
		i3 = i1 + 1;							// nastepna linia
		if(i3 > 9)
			i3 = 0;
		
		bezier[i1][MX] = 128 - (signed char)(bezier[i1][X] - bezier[i3][X]);
		bezier[i1][MY] = 128 - (signed char)(bezier[i1][Y] - bezier[i3][Y]);
	}
}

void krzywe_beziera(void)
{
	unsigned char i1;
	
	for(i1=0;i1<10;i1++)						// rysuj nowe linie
	{
		wsp[X] = bezier[i1][X];
		wsp[Y] = bezier[i1][Y];
		wsp[MX] = bezier[i1][MX];
		wsp[MY] = bezier[i1][MY];
		
		rysuj_linie(0,0);
	}
}*/
//================================================================


//================================================================
//===== Przerwanie od klawiatury =================================
//================================================================
/*ISR(PCINT0_vect)
{
	int i1;
	unsigned char i2=0;
	
	//cli();									// wylacz przerwania
	STER = BLANK;								// wygaszenie osi Z
	
	// dopisac wysylanie czasu przez I2C (po pewnym czasie na timerze)
	// zegar cyfrowy uaktualniany w przerwaniu "1Hz"
	switch(PINA & 31)							// odczyt stanu PA0 - PA4 
	{
		case 28:								// PA0 i PA1 = L
			if(czas[0] == 1)					// zwieksz godziny10
				czas[0] = 0;
			else
				czas[0] = 1;
			break;
		
		case 26:								// PA0 i PA2 = L
			czas[1]++;							// zwieksz godziny1
			if(czas[0] == 0)
			{
				if(czas[1] > 9)
					czas[1] = 0;
			}
			else
			{
				if(czas[1] > 2)
					czas[1] = 0;
			}
			break;
		
		case 22:								// PA0 i PA3 = L
			czas[2]++;							// zwieksz minuty10
			if(czas[2] > 5)
				czas[2] = 0;
			break;
		
		case 14:								// PA0 i PA4 = L
			czas[3]++;							// zwieksz minuty1
			if(czas[3] > 9)
				czas[3] = 0;
			break;
		
		case 29:
		case 27:
		case 23:
		case 15:
			if(co_wyswietlic == 0)						
				co_wyswietlic = 1;				// wlacz zegar cyfrowy
			else
				co_wyswietlic = 0;				// wlacz zegar analogowy
			break;
		
		default: 
			i2 = 1; 
			break;
	}

	if(i2 == 0)
	{
		wyslij_godzine();
		for(i1=0;i1<10000;i1++)					// opoznienie niwelujace drgania stykow
			delay2();
	}

	//sei();									// wlacz przerwania
	GIFR |= 8;									// czysc flage przerwania 
}*/
//================================================================





